@@ -22,6 +22,16 @@
 /*    Contact: Maxence.Guesdon@inria.fr                                          */
 /*********************************************************************************/
 
+//#define DEBUG2 1
+//#define DEBUG3 1
+
+//in makefile, or not: #define ODBC2 1
+
+#ifndef ODBC2
+#define OLD_POINTERS 1
+#endif
+
+
 /*---| includes (common) |---*/
 #include <stdlib.h>
 #include <stdio.h>
@@ -120,8 +130,10 @@ void      displayError( HENV   hEnv,
                         int       iLineNum
                       );
 
+void print_sql_info(SQLHDBC hdbc);
+
 /* Constructeurs des types abstraits ODBC */
-value value_HENV_c () 
+value value_HENV_c ()
 {
   return(Val_int(SQL_NULL_HENV));
 }
@@ -131,6 +143,8 @@ value value_HDBC_c ()
   return(Val_int(SQL_NULL_HDBC));
 }
 
+
+
 /*-----------------------------------------------------------------------------
  * initDB_c
  *-----------------------------------------------------------------------------
@@ -151,7 +165,7 @@ value value_HDBC_c ()
  *-----------------------------------------------------------------------------
  */
 value initDB_c( value v_nom_base, value v_nom_user, value v_password)
-{ 
+{
   CAMLparam3 (v_nom_base, v_nom_user, v_password);
   CAMLlocal1 (res);
 
@@ -210,7 +224,7 @@ value initDB_c( value v_nom_base, value v_nom_user, value v_password)
     Field(res,0) = Val_int ((int) result);
     Field(res,1) = Val_long ((long) SQL_NULL_HENV);
     Field(res,2) = Val_long ((long) SQL_NULL_HDBC);
-    CAMLreturn(res);  
+    CAMLreturn(res);
   }
 
   /*
@@ -227,7 +241,7 @@ value initDB_c( value v_nom_base, value v_nom_user, value v_password)
     Field(res,0) = Val_int ((int) result);
     Field(res,1) = Val_long ((long) SQL_NULL_HENV);
     Field(res,2) = Val_long ((long) SQL_NULL_HDBC);
-    CAMLreturn(res);  
+    CAMLreturn(res);
   }
 
   /*
@@ -254,7 +268,7 @@ value initDB_c( value v_nom_base, value v_nom_user, value v_password)
     Field(res,0) = Val_int ((int) result);
     Field(res,1) = Val_long ((long) SQL_NULL_HENV);
     Field(res,2) = Val_long ((long) SQL_NULL_HDBC);
-    CAMLreturn(res);  
+    CAMLreturn(res);
   }
 
   Field(res,0) = Val_int ((int) 0);
@@ -264,6 +278,122 @@ value initDB_c( value v_nom_base, value v_nom_user, value v_password)
 }
 
 
+#define MAXBUFLEN 1024
+
+/*-----------------------------------------------------------------------------
+ * initDB_driver_c
+ *-----------------------------------------------------------------------------
+ * function: initialisation of DB access
+ * input:    char *connect_string                               (not null)
+ *             pointer to ODBC driver connection string.  The string is driver
+ *             dependent.  It includes the username and password if applicable.
+ *           int prompt
+ *             != 0 if the driver should raise a dialog box to request username
+ *                  and password
+ *             == 0 if the driver should not prompt
+ * output:   int
+ *             != 0 - error
+ *             == 0 - no error
+ *           HENV *phEnv
+ *             pointer to DB environment
+ *           HDBC *phDbc
+ *             pointer to DB context
+ *-----------------------------------------------------------------------------
+ */
+value initDB_driver_c( value v_connect_string, value v_prompt)
+{
+  CAMLparam2 (v_connect_string,v_prompt);
+  CAMLlocal1 (res);
+  res =  alloc_tuple(3);
+
+  char *connect_string = String_val(v_connect_string);
+  int prompt = Bool_val(v_prompt);
+  RETCODE result;
+  //HENV *phEnv = SQL_NULL_HENV;
+  //HDBC *phDbc = SQL_NULL_HDBC;
+
+  SQLHENV      henv = SQL_NULL_HENV;
+  SQLHDBC      hdbc1 = SQL_NULL_HDBC;
+  SQLHSTMT      hstmt1 = SQL_NULL_HSTMT;
+
+  SQLCHAR      ConnStrIn[MAXBUFLEN] = "";
+
+  SQLCHAR      ConnStrOut[MAXBUFLEN];
+  SQLSMALLINT   cbConnStrOut = 0;
+
+  SQLRETURN cliRC = SQL_SUCCESS;
+
+  /* allocate an environment handle */
+  cliRC = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv);
+  if (cliRC != SQL_SUCCESS)
+  {
+    printf("\n--ERROR while allocating the environment handle.\n");
+    printf("  cliRC = %d\n", cliRC);
+    printf("  line  = %d\n", __LINE__);
+    printf("  file  = %s\n", __FILE__);
+    fflush(stdout);
+
+    Field(res,0) = Val_int (-17);
+    Field(res,1) = Val_long ((long) henv);
+    Field(res,2) = Val_long ((long) hdbc1);
+    CAMLreturn(res);
+  }
+
+  /* set attribute to enable application to run as OCBC 3.0 application */
+#ifdef ODBC2
+  cliRC = SQLSetEnvAttr(henv,
+                     SQL_ATTR_ODBC_VERSION,
+                     (void *)SQL_OV_ODBC2,
+                     0);
+#endif
+
+  //ENV_HANDLE_CHECK(henv, cliRC);
+
+  cliRC = SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1);
+  if (cliRC != SQL_SUCCESS)
+  {
+    printf("\n--ERROR while allocating the environment handle.\n");
+    printf("  cliRC = %d\n", cliRC);
+    printf("  line  = %d\n", __LINE__);
+    printf("  file  = %s\n", __FILE__);
+    fflush(stdout);
+
+    Field(res,0) = Val_int (-18);
+    Field(res,1) = Val_long ((long) henv);
+    Field(res,2) = Val_long ((long) hdbc1);
+    CAMLreturn(res);
+  }
+
+  // Make connection without data source. Ask that driver
+  // prompt if insufficient information. Driver returns
+  // SQL_ERROR and application prompts user
+  // for missing information. Window handle not needed for
+  // SQL_DRIVER_NOPROMPT.
+  result = SQLDriverConnect(hdbc1,      // Connection handle
+                  NULL,         // Window handle
+                  connect_string,      // Input connect string
+                  SQL_NTS,         // Null-terminated string
+                  ConnStrOut,      // Address of output buffer
+                  MAXBUFLEN,      // Size of output buffer
+                  &cbConnStrOut,   // Address of output length
+                  (prompt ? SQL_DRIVER_PROMPT : SQL_DRIVER_NOPROMPT));
+
+  if(result==SQL_SUCCESS_WITH_INFO)
+    print_sql_info(hdbc1);
+
+  Field(res,0) = Val_int (result);
+  if(result==SQL_SUCCESS_WITH_INFO) {Field(res,0) = Val_int (0);}
+  if(result==SQL_NO_DATA) {Field(res,0) = Val_int (-11);}
+  if(result==SQL_ERROR) {Field(res,0) = Val_int (-12);}
+  if(result==SQL_INVALID_HANDLE) {Field(res,0) = Val_int (-13);}
+
+  Field(res,1) = Val_long ((long) henv);
+  Field(res,2) = Val_long ((long) hdbc1);
+  //printf("henv=%0x, hdbc=%0x\n",henv,hdbc1); fflush(stdout);
+  CAMLreturn(res);
+}
+
+
 /*-----------------------------------------------------------------------------
  * exitDB
  *-----------------------------------------------------------------------------
@@ -278,11 +408,11 @@ value initDB_c( value v_nom_base, value v_nom_user, value v_password)
  *-----------------------------------------------------------------------------
  */
 value exitDB_c( value v_phEnv, value v_phDbc)
-{ 
+{
   CAMLparam2 (v_phEnv, v_phDbc);
   CAMLlocal1 (res);
-  HENV *phEnv = (HENV *) (Long_val(v_phEnv));
-  HDBC *phDbc = (HDBC *) (Long_val(v_phDbc));
+  HENV *phEnv = (HENV *) (Unsigned_long_val(v_phEnv));
+  HDBC *phDbc = (HDBC *) (Unsigned_long_val(v_phDbc));
   RETCODE result;
 
 #ifdef DEBUG2
@@ -298,22 +428,37 @@ value exitDB_c( value v_phEnv, value v_phDbc)
 #endif
     CAMLreturn (Val_int ((int)-1));
   }
-   /*
+
+#ifdef DEBUG3
+  printf("<1>"); fflush(stdout);
+#endif
+  /*
   ** commit transactions
   */
+#ifdef OLD_POINTERS
   result = SQLTransact( *phEnv, *phDbc, SQL_COMMIT );
+#else
+  result = SQLTransact( phEnv, phDbc, SQL_COMMIT );
+#endif
   if( SQL_SUCCESS != result ) {
 #ifdef DEBUG2
     printf("Erreur SQLTransact\n");
     fflush(stdout);
     displayError( *phEnv, *phDbc, SQL_NULL_HENV, result, __LINE__ );
 #endif
-  }
+}
 
   /*
   ** disconnect from DB
   */
+#ifdef DEBUG3
+  printf("<2>"); fflush(stdout);
+#endif
+#ifdef OLD_POINTERS
   result = SQLDisconnect( *phDbc );
+#else
+  result = SQLDisconnect( phDbc );
+#endif
   if( SQL_SUCCESS != result ) {
 #ifdef DEBUG2
     printf("Erreur SQLDisconnect\n");
@@ -325,7 +470,14 @@ value exitDB_c( value v_phEnv, value v_phDbc)
   /*
   ** free connection to DB
   */
+#ifdef DEBUG3
+  printf("<3>"); fflush(stdout);
+#endif
+#ifdef OLD_POINTERS
   result = SQLFreeConnect( *phDbc );
+#else
+  result = SQLFreeConnect( phDbc );
+#endif
   if( SQL_SUCCESS != result ) {
 #ifdef DEBUG2
     printf("Erreur SQLFreeConnect\n");
@@ -339,7 +491,14 @@ value exitDB_c( value v_phEnv, value v_phDbc)
   /*
   ** free environment
   */
+#ifdef DEBUG3
+  printf("<4>"); fflush(stdout);
+#endif
+#ifdef OLD_POINTERS
   result = SQLFreeEnv( *phEnv );
+#else
+  result = SQLFreeEnv( phEnv );
+#endif
   if( SQL_SUCCESS != result ) {
 #ifdef DEBUG2
     printf("Erreur SQLFreeEnv\n");
@@ -349,7 +508,11 @@ value exitDB_c( value v_phEnv, value v_phDbc)
   }
   else
     *phEnv = SQL_NULL_HENV;
- 
+
+#ifdef DEBUG3
+  printf("<5>"); fflush(stdout);
+#endif
+
   res = Val_int ((int) 0);
   CAMLreturn(res);
 }
@@ -390,12 +553,14 @@ env * new_env (void) {
   q_env->exec_pData[0] = NULL;
   q_env->phEnv = NULL;
   q_env->phDbc = NULL;
-  
-  return q_env ;  
+
+  return q_env ;
 }
 
+
+
 value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
-{ 
+{
   CAMLparam3(v_phEnv, v_phDbc, v_cmd);
   CAMLlocal1 (exec_res);
   CAMLlocal1 (caml_q_env) ;
@@ -415,19 +580,24 @@ value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
   /*  caml_q_env = alloc (sizeof(env*), Abstract_tag);
   Store_field (caml_q_env, 0, (value) q_env);
   */
-  q_env->phEnv = (HENV *) (Long_val(v_phEnv));
-  q_env->phDbc = (HDBC *) (Long_val(v_phDbc));
+  q_env->phEnv = (HENV *) (Unsigned_long_val(v_phEnv));
+  q_env->phDbc = (HDBC *) (Unsigned_long_val(v_phDbc));
+  //printf("phEnv=%0x, phDbc=%0x\n",q_env->phEnv,q_env->phDbc); fflush(stdout);
 
 #ifdef DEBUG2
   printf("Appel de execDB cmd : %s\n", cmd);
   fflush(stdout);
 #endif
 
+#ifdef DEBUG3
+   printf("<1>"); fflush(stdout);
+#endif
+
   /*
   ** check parameter list
   */
-  if( '\0' == cmd[0] 
-      || q_env->phEnv == SQL_NULL_HENV 
+  if( '\0' == cmd[0]
+      || q_env->phEnv == SQL_NULL_HENV
       || q_env->phDbc == SQL_NULL_HDBC ) {
 #ifdef DEBUG2
     printf("Erreur parametres\n");
@@ -438,20 +608,36 @@ value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
     retour = alloc_tuple (2) ;
     Store_field (retour, 0, exec_res);
     Store_field (retour, 1, caml_q_env);
-    CAMLreturn(retour);  
+    CAMLreturn(retour);
   }
+#ifdef DEBUG3
+  printf("<2>"); fflush(stdout);
+#endif
 
   /*
   ** get statement handle
   */
+
+#ifdef ODBC2
+#ifdef DEBUG3
+  int x = (int)(*(q_env->phDbc));
+  printf("<2.5,%0x>",x); fflush(stdout);
+#endif
+  result = SQLAllocHandle( SQL_HANDLE_STMT,
+			   (q_env->phDbc), &(q_env->exec_hstmt) );
+#else
   result = SQLAllocStmt( *(q_env->phDbc), &(q_env->exec_hstmt) );
+#endif
+#ifdef DEBUG3
+  printf("<3>"); fflush(stdout);
+#endif
   if( SQL_SUCCESS != result ) {
 #ifdef DEBUG2
     printf("Erreur SQLAllocStmt\n");
     fflush(stdout);
-    displayError( *(q_env->phEnv), 
-		  *(q_env->phDbc), 
-		  q_env->exec_hstmt, 
+    displayError( *(q_env->phEnv),
+		  *(q_env->phDbc),
+		  q_env->exec_hstmt,
 		  result, __LINE__ );
 #endif
     /* Allocation de la structure de retour */
@@ -470,9 +656,9 @@ value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
     printf("Erreur SQLPrepare\n");
     printf("%s\n", cmd);
     fflush(stdout);
-    displayError( *(q_env->phEnv), 
-		  *(q_env->phDbc), 
-		  q_env->exec_hstmt, 
+    displayError( *(q_env->phEnv),
+		  *(q_env->phDbc),
+		  q_env->exec_hstmt,
 		  result, __LINE__ );
 #endif
     /* Allocation de la structure de retour */
@@ -486,13 +672,17 @@ value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
   /*
   ** execute statement
   */
-  if( SQL_SUCCESS != (result=SQLExecute(q_env->exec_hstmt)) ) {
+  result=SQLExecute(q_env->exec_hstmt);
+  if( result==SQL_SUCCESS) {}
+  else if(result==SQL_SUCCESS_WITH_INFO)
+    print_sql_info(q_env->phDbc);
+  else {
 #ifdef DEBUG2
     printf("Erreur SQLExecute\n");
     fflush(stdout);
-    displayError( *(q_env->phEnv), 
-		  *(q_env->phDbc), 
-		  q_env->exec_hstmt, 
+    displayError( *(q_env->phEnv),
+		  *(q_env->phDbc),
+		  q_env->exec_hstmt,
 		  result, __LINE__ );
 #endif
     /* Allocation de la structure de retour */
@@ -506,55 +696,55 @@ value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
   ** get number of rows affected / columns returned
   */
   q_env->exec_iRowCount = 0;
-  result = SQLRowCount(q_env->exec_hstmt, 
+  result = SQLRowCount(q_env->exec_hstmt,
 		       (SQLINTEGER FAR *) &(q_env->exec_iRowCount) );
-#ifdef DEBUG2
+#ifdef DEBUG_LIGHT
   printf( "number of rows affected    : %d\n",
           (SQL_SUCCESS == result) ? q_env->exec_iRowCount
                                : -1
-        );
+	  );
 #endif
   q_env->exec_iResColumns = 0;
   result = SQLNumResultCols(q_env->exec_hstmt,
 			    (SWORD FAR *) &(q_env->exec_iResColumns) );
-#ifdef DEBUG2
+#ifdef DEBUG_LIGHT
   printf( "number of columns returned : %d\n",
           (SQL_SUCCESS == result) ? q_env->exec_iResColumns
                                : -1
-        );
+	  ); fflush(stdout);
 #endif
 
   /*
   ** get table description and create list with pointers
   */
   if( 0 < q_env->exec_iResColumns )
-  { 
+  {
 #ifdef DEBUG2
     printf( "result columns:\n" );
 #endif
-    for( exec_ci = q_env->exec_iResColumns; 
-	 exec_ci >=1; 
+    for( exec_ci = q_env->exec_iResColumns;
+	 exec_ci >=1;
 	 exec_ci-- )
-      { 
-	if( SQL_SUCCESS != (result=SQLDescribeCol( q_env->exec_hstmt, 
-						   exec_ci, 
+      {
+	if( SQL_SUCCESS != (result=SQLDescribeCol( q_env->exec_hstmt,
+						   exec_ci,
 						   &(exec_szColName[0]),
 						   sizeof(exec_szColName) - 1,
-						   &(exec_cbColName), 
+						   &(exec_cbColName),
 						   &(exec_fColType),
 						   (UDWORD FAR *) &(exec_uiColPrecision),
-						   &(exec_iColScaling), 
+						   &(exec_iColScaling),
 						   &(exec_fColNullable)
 						   )
 			    )
 	    )
-	  { 
+	  {
 #ifdef DEBUG2
 	    printf("Erreur SQLDescribeCol\n");
 	    fflush(stdout);
-	    displayError( *(q_env->phEnv), 
-			  *(q_env->phDbc), 
-			  q_env->exec_hstmt, 
+	    displayError( *(q_env->phEnv),
+			  *(q_env->phDbc),
+			  q_env->exec_hstmt,
 			  result, __LINE__ );
 #endif
 	    exec_res = Val_int(result);
@@ -575,11 +765,11 @@ value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
 	else
 	  {
 	    memset( (q_env->exec_pData)[exec_ci], 0, (exec_uiColPrecision) +1 );
-	    result = SQLBindCol( q_env->exec_hstmt, 
-				 exec_ci, 
-				 SQL_C_CHAR, 
+	    result = SQLBindCol( q_env->exec_hstmt,
+				 exec_ci,
+				 SQL_C_CHAR,
 				 (q_env->exec_pData)[exec_ci],
-				 (exec_uiColPrecision)+1, 
+				 (exec_uiColPrecision)+1,
 				 &(q_env->exec_indicator[exec_ci])
 				 );
 	  }
@@ -596,7 +786,7 @@ value execDB_c( value v_phEnv, value v_phDbc, value v_cmd)
       CAMLreturn(retour);
     }
   else
-    { 
+    {
       exec_res = Val_int(1);
       retour = alloc_tuple (2) ;
       Store_field (retour, 0, exec_res);
@@ -626,7 +816,7 @@ value itere_execDB_c (value caml_q_env, value nb_records_ml) {
 
 #ifdef DEBUG2
   printf( "itere_execDB_c start\n");
-#endif     
+#endif
 
   exec_l_head2 = Val_int(0);
   exec_string_list_list = exec_l_head2;
@@ -634,13 +824,13 @@ value itere_execDB_c (value caml_q_env, value nb_records_ml) {
 
 #ifdef DEBUG2
   printf( "nb_records = %d\n", nb_records );
-#endif     
+#endif
 
   if (0 < q_env->exec_iResColumns )
     {
 #ifdef DEBUG2
       printf( " 0 < q_env->exec_iResColumns \n" );
-#endif      
+#endif
       while (i < nb_records) {
 	if (! ( SQL_SUCCESS           == (result=SQLFetch(q_env->exec_hstmt)) ||
 		SQL_SUCCESS_WITH_INFO == result ))
@@ -669,12 +859,12 @@ value itere_execDB_c (value caml_q_env, value nb_records_ml) {
 		}
 		else {
 #ifdef DEBUG2
-		  printf( "'%s' ", (NULL == q_env->exec_pData[exec_ci]) ? 
+		  printf( "'%s' ", (NULL == q_env->exec_pData[exec_ci]) ?
 			  "<ERROR>" : q_env->exec_pData[exec_ci] );
 		  fflush( stdout );
 #endif
-		  Store_field(exec_temp, 0, 
-			      copy_string((NULL == q_env->exec_pData[exec_ci]) ? 
+		  Store_field(exec_temp, 0,
+			      copy_string((NULL == q_env->exec_pData[exec_ci]) ?
 					  "<ERROR>" : q_env->exec_pData[exec_ci]));
 		}
 	      } /* for */
@@ -683,7 +873,7 @@ value itere_execDB_c (value caml_q_env, value nb_records_ml) {
 #endif
 	    exec_string_list = exec_l_head;
 
-	    exec_l_head2 = alloc_tuple (2);	    
+	    exec_l_head2 = alloc_tuple (2);
 	    Store_field (exec_l_head2, 0, exec_string_list);
 	    Store_field (exec_l_head2, 1, Val_int(0));
 
@@ -704,7 +894,7 @@ value itere_execDB_c (value caml_q_env, value nb_records_ml) {
     } /* if */
   else
     {}
-  
+
   /*
   ** display last return code
   */
@@ -727,7 +917,7 @@ value itere_execDB_c (value caml_q_env, value nb_records_ml) {
   exec_res = alloc_tuple(2);
   Store_field(exec_res,0,Val_int (i));
 #ifdef DEBUG2
-  printf("avant retour de itere_2bis\n"); 
+  printf("avant retour de itere_2bis\n");
 #endif
   Store_field(exec_res,1, exec_string_list_list);
 #ifdef DEBUG2
@@ -767,7 +957,7 @@ int get_OCaml_SQL_type_code (int code)
 pour chaque champ retourn par la dernire requte excute
 et non encore libre. */
 value get_infoDB_c(value caml_q_env, value v_phEnv, value v_phDbc)
-{ 
+{
   CAMLparam2 (v_phEnv, v_phDbc);
   CAMLlocal2 (info_temp, info_l_head);
   CAMLlocal1 (info_cpl);
@@ -798,20 +988,20 @@ value get_infoDB_c(value caml_q_env, value v_phEnv, value v_phDbc)
 #endif
     /* On retourne une liste vide */
     info_l_head = Val_int (0);
-    CAMLreturn(info_l_head);  
+    CAMLreturn(info_l_head);
   }
 
   /*
   ** get table description and create list with pointers
   */
   if( 0 < q_env->exec_iResColumns )
-  { 
+  {
 #ifdef DEBUG2
     printf( "result columns:\n" );
 #endif
     /* Initialisation de la liste des descriptions de colonnes */
     info_l_head = Val_int(0);
-    
+
     for( exec_ci = q_env->exec_iResColumns; exec_ci >=1; exec_ci-- )
     { /*
       ** display table info
@@ -819,29 +1009,29 @@ value get_infoDB_c(value caml_q_env, value v_phEnv, value v_phDbc)
 #ifdef DEBUG2
       printf( "  [%03u] ", exec_ci ); fflush( stdout );
 #endif
-      if( SQL_SUCCESS != (result=SQLDescribeCol(q_env->exec_hstmt, 
-						exec_ci, 
+      if( SQL_SUCCESS != (result=SQLDescribeCol(q_env->exec_hstmt,
+						exec_ci,
 						&(exec_szColName[0]),
 						sizeof(exec_szColName) - 1,
-						&(exec_cbColName), 
+						&(exec_cbColName),
 						&(exec_fColType),
 						(UDWORD FAR *) &(exec_uiColPrecision),
-						&(exec_iColScaling), 
+						&(exec_iColScaling),
 						&(exec_fColNullable)
                                             )
                           )
-         ) { 
+         ) {
 #ifdef DEBUG2
 	printf("Erreur SQLDescribeCol\n");
 	fflush(stdout);
-	displayError( *(q_env->phEnv), 
-		      *(q_env->phDbc), 
-		      q_env->exec_hstmt, 
+	displayError( *(q_env->phEnv),
+		      *(q_env->phDbc),
+		      q_env->exec_hstmt,
 		      result, __LINE__ );
 #endif
         /* Allocation de la structure de retour */
 	info_l_head = Val_int(0);
-	CAMLreturn(info_l_head);  
+	CAMLreturn(info_l_head);
       }
 
 #ifdef DEBUG2
@@ -900,7 +1090,7 @@ value get_infoDB_c(value caml_q_env, value v_phEnv, value v_phDbc)
 }
 
 
-/* free_ExecDB_c : fonction de dsallocation des structures alloues par 
+/* free_ExecDB_c : fonction de dsallocation des structures alloues par
    execDB_c */
 void free_execDB_c (value caml_q_env) {
   int result;
@@ -909,14 +1099,14 @@ void free_execDB_c (value caml_q_env) {
   env* q_env = (env*) caml_q_env ;
 #ifdef DEBUG2
   printf( "free_execDB_c start\n");
-#endif   
+#endif
     /*
     ** free allocated memory
     */
   for( exec_ci = 1 ;
-	 exec_ci <= q_env->exec_iResColumns; 
+	 exec_ci <= q_env->exec_iResColumns;
 	 exec_ci++ )
-    { 
+    {
 #ifdef DEBUG2
       fprintf(stderr, "free for exe_ci = %d\n", exec_ci);
       fflush(stderr);
@@ -951,10 +1141,10 @@ void free_execDB_c (value caml_q_env) {
     printf("Erreur SQLFreeStmt\n");
     fflush(stdout);
     displayError( *(q_env->phEnv),
-		  *(q_env->phDbc), 
-		  q_env->exec_hstmt, 
+		  *(q_env->phDbc),
+		  q_env->exec_hstmt,
 		  result, __LINE__ );
-    
+
 #endif
   }
   free(q_env);
@@ -995,9 +1185,9 @@ void displayError( HENV hEnv, HDBC hDbc, HSTMT hStmt,
   ** display native error code
   */
   /* fprintf( stderr, "\a-----------------------\n" ); */
-  fprintf( stderr, "-----------------------\n" );
-  fprintf( stderr, "SQL error              : %d\n", iRC );
-  fprintf( stderr, "line number            : %d\n", iLineNum );
+  fprintf( stdout, "-----------------------\n" );
+  fprintf( stdout, "SQL error              : %d\n", iRC );
+  fprintf( stdout, "line number            : %d\n", iLineNum );
 
   /*
   ** display all error messages corresponding to this code
@@ -1008,12 +1198,36 @@ void displayError( HENV hEnv, HDBC hDbc, HSTMT hStmt,
                                 )
        )
   {
-    fprintf( stderr, "SQL state              : %s\n", szSqlState );
-    fprintf( stderr, "native error code      : %ld\n", iSqlCode );
-    fprintf( stderr, "%s\n", szBuffer );
+    fprintf( stdout, "SQL state              : %s\n", szSqlState );
+    fprintf( stdout, "native error code      : %ld\n", iSqlCode );
+    fprintf( stdout, "%s\n", szBuffer );
   } /* while */
 
   fprintf( stderr, "-----------------------\n" );
   /* fprintf( stderr, "\a-----------------------\n" ); */
-
+  fflush(stderr);
+  fflush(stdout);
 } /* displayError */
+
+
+void print_sql_info(SQLHDBC hdbc)
+{
+  SQLRETURN res2;
+  SQLCHAR state = 0;
+  SQLINTEGER nativeState = 0;
+  SQLCHAR msg[MAXBUFLEN+1];
+  SQLSMALLINT msgLength = 0;
+  do {
+    res2 = SQLGetDiagRec(SQL_HANDLE_DBC,
+			 hdbc,
+			 1, //SQLSMALLINT     RecNumber,
+			 &state, //SQLCHAR *     Sqlstate,
+			 &nativeState, //SQLINTEGER *     NativeErrorPtr,
+			 msg, // SQLCHAR *     MessageText,
+			 MAXBUFLEN, // SQLSMALLINT     BufferLength,
+			 &msgLength //SQLSMALLINT *     TextLengthPtr
+			 );
+    printf("state=%0xh, nativeState=%0xh, msg=%s\n", state, nativeState, msg);
+  } while(res2==SQL_SUCCESS_WITH_INFO);;
+  fflush(stdout);
+}
