@@ -1,5 +1,3 @@
-(* vim:ts=8
-*)
 (*********************************************************************************)
 (*                OCamlODBC                                                         *)
 (*                                                                               *)
@@ -30,7 +28,7 @@
 let logiciel = "OCamlODBC"
 
 (** The software version *)
-let version = "2.9"
+let version = "2.10"
 
 exception SQL_Error of string
 
@@ -91,6 +89,7 @@ module OCamlODBC_messages =
   struct
     let disconnect = "ODBC : problem while disconnecting"
     let connection nom_base nom_user pzPasswd iRC1 = "Error while connecting to database "^nom_base^" as "^nom_user^" with password <"^pzPasswd^"> : "^(string_of_int iRC1)
+    let connection_driver connect_string iRC1 = "Error while connecting to database with connection string "^connect_string^"> : "^(string_of_int iRC1)
   end
 
 type connection =
@@ -115,6 +114,19 @@ let connect base user passwd =
   else
     raise (SQL_Error (OCamlODBC_messages.connection base user passwd iRC1))
 
+let connect_driver ?(prompt=false) connect_string =
+  let (iRC1,hEnv,pHDbc) = SQLInterface.initDB_driver connect_string prompt in
+  if (iRC1 = 0) then 
+    {
+      phEnv = hEnv;
+      phDbc = pHDbc;
+      base = connect_string ;
+      user = "" ;
+      passwd = "" ;
+    } 
+  else
+    raise (SQL_Error (OCamlODBC_messages.connection_driver connect_string iRC1))
+
 let disconnect connection =
   let iRC = (SQLInterface.exitDB connection.phEnv connection.phDbc) in
   if (iRC > 0) then
